// Aqsis
// Copyright  1997 - 2001, Paul C. Gregory
//
// Contact: pgregory@aqsis.com
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


/** \file
		\brief Implements CqSemaphire class which encapsulates the system specific semaphore.
		\author Paul C. Gregory (pgregory@aqsis.com)
*/

#include	<windows.h>

#include	"aqsis.h"
#include	"semaphore.h"

//--------------------------------------------------------------------------
/** Constructor.
 */

CqSemaphore::CqSemaphore() :
		m_pNativeSemaphore( CreateSemaphore( NULL, 0, 32767, NULL ) )
{}


//--------------------------------------------------------------------------
/** Destructor.
 */

CqSemaphore::~CqSemaphore()
{
	CloseHandle( m_pNativeSemaphore );
}


//--------------------------------------------------------------------------
/** Increase the semaphore count by one (potentially releasing a blocked thread)
 */

void
CqSemaphore::Signal()
{
	ReleaseSemaphore( m_pNativeSemaphore, 1, NULL );
}


//--------------------------------------------------------------------------
/** Decrease the semaphore count by one (may block)
 */

void
CqSemaphore::Wait()
{
	WaitForSingleObject( m_pNativeSemaphore, INFINITE );
}

